variable "output_directory" {
  type    = string
  default = "output"
}

variable "checksum_directory" {
  type    = string
  default = "checksums"
}

variable "vm_name" {
  type    = string
  default = "debian-12"
}

variable "iso_url" {
  type    = string
  default = "https://cdimage.debian.org/debian-cd/current/amd64/iso-cd/debian-12.8.0-amd64-netinst.iso"
}

variable "iso_checksum" {
  type    = string
  default = "sha256:04396d12b0f377958a070c38a923c227832fa3b3e18ddc013936ecf492e9fbb3"
}

variable "cpus" {
  type    = number
  default = 1
}

variable "memsize" {
  type    = number
  default = 1024
}

variable "qemu_accelerator" {
  type    = string
  default = "kvm"
}

variable "disk_size" {
  type    = string
  default = "8G"
}

variable "disk_detect_zeroes" {
  type    = string
  default = "on"
}

variable "disk_compression" {
  type    = bool
  default = false
}

variable "disk_format" {
  type    = string
  default = "qcow2"
}

variable "username" {
  type    = string
  default = "student"
}

variable "password" {
  type    = string
  default = "student"
}

variable "headless" {
  type    = bool
  default = false
}

variable "img_name" {
  type    = string
  default = "img"
}

source "qemu" "debian12" {
  iso_url             = var.iso_url
  iso_checksum        = var.iso_checksum
  output_directory    = "${var.output_directory}/${var.vm_name}"
  cpus                = var.cpus
  memory              = var.memsize
  disk_size           = var.disk_size
  disk_interface      = "virtio"
  disk_compression    = var.disk_compression
  disk_detect_zeroes  = var.disk_detect_zeroes
  format              = var.disk_format
  http_content        = {
    "/debian-12-preseed.cfg" = templatefile("scripts/autoinst/debian-12-preseed.cfg", {
      user = {
        username = var.username
        password = var.password
      }
    })
  }
  accelerator       = var.qemu_accelerator
  ssh_username      = var.username
  ssh_password      = var.password
  ssh_timeout       = "30m"
  shutdown_command  = "rm -rf ~/.ansible && echo '${var.password}' | sudo -S poweroff"
  vm_name           = "${var.img_name}"
  net_device        = "virtio-net"
  headless          = var.headless
  boot_wait         = "1s"
  boot_command      = [
    "<tab> ",
    "hostname=${var.vm_name}.local ",
    "auto=true priority=critical ",
    "preseed/url=http://{{ .HTTPIP }}:{{ .HTTPPort }}/debian-12-preseed.cfg<enter>"
  ]
  boot_key_interval = "50ms"
}

build {
  sources = ["source.qemu.debian12"]

  provisioner "ansible" {
    playbook_file    = "scripts/ansible/debian-12.yml"
    user             = var.username
    use_proxy        = false
    extra_arguments  = [
      "--extra-vars", "ansible_password='${var.password}' ansible_become_pass='${var.password}'",
      "--extra-vars", "hostname='${var.vm_name}'",
    ]
  }

  post-processor "shell-local" {
    inline = ["rm -f ${var.checksum_directory}/${var.vm_name}.*"]
  }

  post-processor "shell-local" {
      inline = ["qemu-img snapshot -c new '${var.output_directory}/${var.vm_name}/${var.img_name}'"]
  }

  post-processor "checksum" {
    checksum_types = ["sha256", "sha512"]
    output = "${var.checksum_directory}/${var.vm_name}.{{.ChecksumType}}"
  }

  post-processor "shell-local" {
    inline = ["sed -Ei 's/${var.img_name}([^ ]*)$/${var.vm_name}.${var.disk_format}\\1/g' ${var.checksum_directory}/${var.vm_name}.*"]
  }
}
