#!/bin/bash

set -eu -o pipefail

script=$(readlink -e "$0")
root_dir=$(dirname "${script}")
output_root_dir="${root_dir}/output"
output_baseimg_dir="${output_root_dir}/base_images"
output_labs_dir="${output_root_dir}/labs"
vm_dir=$(readlink -e "${root_dir}/../vms")
checksums_dir=$(readlink -e "${root_dir}/../checksums")
scripts_dir=$(readlink -e "${root_dir}/../scripts")
runvm_script="${scripts_dir}/shell/runvm.sh"

force=0
pack_base=0
pack_lab=0

function escape_paths() {
    printf "%q\n" "$@"
}

declare -A extra_files
extra_files=(
    [lab-routing]=$(escape_paths "${scripts_dir}/shell/lab-routing"/*)
)

mkdir -p "${output_baseimg_dir}" "${output_labs_dir}"

function err() {
    echo "$@" >&2
}

function usage() {
    err "Usage:"
    err "Package everything:"
    err "    $0 [options]"
    err "Package only base images (not compressed into archives; only compute checksums):"
    err "    $0 [options] base-images"
    err "Package only lab images:"
    err "    $0 [options] lab"
    err "Package a specific lab's images:"
    err "    $0 [options] lab lab-name"
    err ""
    err "Options:"
    err "    -f: force package (by default only checksums are computed if files exist)"
}

function package_base_images() {
    # Move to the base images directory
    cd "${output_baseimg_dir}"

    for base_img in $(find "${vm_dir}" -maxdepth 1 -name "*.qcow2" -type f); do
        img_name=$(basename "${base_img}")

        # Only create symbolic links for the base images,
        # as they take a lot of space
        [ -e "${img_name}" ] || ln -sv "${base_img}" "${img_name}"
    done
}

function update_base_image_checksums() {
    # Move to the base images directory so paths in the checksums are correct
    cd "${output_baseimg_dir}"
    # Remove old checksum files
    rm -f SHA256SUMS SHA512SUMS

    for base_img in $(find "${vm_dir}" -maxdepth 1 -name "*.qcow2" -type f); do
        img_name=$(basename "${base_img}")
        img_base_name="${img_name%.*}"

        cat "${checksums_dir}/${img_base_name}.sha256" >> SHA256SUMS
        cat "${checksums_dir}/${img_base_name}.sha512" >> SHA512SUMS
    done
}

function package_lab() {
    lab_name="$1"
    zip_name="${lab_name}.zip"

    # Move to the lab directory to allow creating zips with relative paths
    # to contents
    cd "${output_labs_dir}"

    if [ "${force}" -eq 0 -a -f "${zip_name}" ]; then
        return
    fi

    if [ "${force}" -ne 0 ]; then
        rm "${zip_name}"
    fi

    lab_files=( $(find "${vm_dir}" -mindepth 1 -maxdepth 1 -name "${lab_name}*" -type d) )
    if [ ${#lab_files[@]} -gt 0 ]; then
        lab_files+=( "${runvm_script}" )
    fi

    if [[ -v "extra_files[${lab_name}]" ]]; then
        lab_files+=( ${extra_files[${lab_name}]} )
    fi

    if [ ${#lab_files[@]} -eq 0 ]; then
        err "No files associated with lab ${lab_name} found. Aborting..."
        exit 2
    fi

    for lab_file in "${lab_files[@]}"; do
        if [ ! -e "${lab_file}" ]; then
            err "Invalid extra file specified '${lab_file}'. Aborting..."
            exit 3
        fi

        link_name=$(basename "${lab_file}")

        # Create a symbolic link to the lab directory so we get the correct
        # inspired by https://stackoverflow.com/a/72282401
        ln -s "${lab_file}" "${link_name}"
        zip -r "${zip_name}" "${link_name}"
        rm "${link_name}"
    done
}

function package_all_labs() {
    for lab_dir in $(find "${vm_dir}" -mindepth 1 -maxdepth 1 -name "lab-*" -type d); do
        lab_dir_name=$(basename "${lab_dir}")
        lab_name=$(echo "${lab_dir_name}" | sed -E 's/-[0-9]$//')

        package_lab "${lab_name}"
    done
}

function update_lab_checksums() {
    # Move to the lab directory
    cd "${output_labs_dir}"

    # Compute the SHA256 and SHA512 checksums for the lab zips
    # we are still in the lab zip directory
    echo ">>> Computing lab zip checksums..."
    sha256sum *.zip > SHA256SUMS
    sha512sum *.zip > SHA512SUMS
}

while getopts "fh" option; do
    case "${option}" in
        h) usage; exit 0 ;;
        f) force=1 ;;
    esac
done

shift $((OPTIND-1))

if [ $# -gt 2 ]; then
    usage
    exit 1
fi

if [ $# -eq 2 ]; then
    if [ "$1" = "lab" ]; then
        package_lab "$2"
        update_lab_checksums
        exit 0
    fi
fi

if [ $# -eq 0 ]; then
    pack_base=1
    pack_labs=1
elif [ $# -eq 1 ]; then
    case "$1" in
    base-images)
        pack_base=1;;
    lab)
        pack_lab=1;;
    *)
        usage
        exit 1
    esac
fi

if [ "${pack_base}" -eq 1 ]; then
    package_base_images
    update_base_image_checksums
fi

if [ "${pack_labs}" -eq 1 ]; then
    package_all_labs
    update_lab_checksums
fi
